<?php
/**
 * This is a file IO handler. It handles reading from and writing to files.
 * Missing paths in writing process will be automatically created.
 *
 * @author		Roland Haeder <webmaster@mxchange.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link 		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FileIoHandler extends BaseMiddleware {
	/**
	 * The *real* file input class we shall use for reading data
	 */
	private $inputStream = null;

	/**
	 * The *real* file output class we shall use for reading data
	 */
	private $outputStream = null;

	/**
	 * An instance of this class
	 */
	private static $thisInstance = null;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set description
		$this->setObjectDescription("Datei-Ein-/Ausgabe-Handler");

		// Create an unique ID
		$this->createUniqueID();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();

		// Set own instance
		self::$thisInstance = $this;
	}

	/**
	 * Creates an instance of this class and prepares the IO system. This is
	 * being done by setting the default file IO class
	 *
	 * @return	$ioInstance	A prepared instance of FilIOHandler
	 */
	public final static function createFileIoHandler () {
		// Get instance
		$ioHandler = new FileIoHandler();

		// Set the *real* file IO instances (both the same)
		$ioHandler->setInputStream(FileIoStream::createFileIoStream());
		$ioHandler->setOutputStream(FileIoStream::createFileIoStream());

		// Return instance
		return $ioHandler;
	}

	/**
	 * Getter for an instance of this class
	 *
	 * @return	$thisInstance	An instance of this class
	 */
	public final static function getInstance () {
		return self::$thisInstance;
	}

	/**
	 * Setter for the *real* file input instance
	 *
	 * @param		$inputStream	The *real* file-input class
	 */
	public final function setInputStream (FileInputStreamer $inputStream) {
		$this->inputStream = $inputStream;
	}

	/**
	 * Getter for the *real* file input instance
	 *
	 * @return	$inputStream	The *real* file-input class
	 */
	public final function getInputStream () {
		return $this->inputStream;
	}

	/**
	 * Setter for the *real* file output instance
	 *
	 * @param		$outputStream	The *real* file-output class
	 */
	public final function setOutputStream (FileOutputStreamer $outputStream) {
		$this->outputStream = $outputStream;
	}

	/**
	 * Getter for the *real* file output instance
	 *
	 * @return	$outputStream	The *real* file-output class
	 */
	public final function getOutputStream () {
		return $this->outputStream;
	}

	/**
	 * Saves a file with data by using the current output stream
	 *
	 * @see	FileOutputStreamer
	 */
	public function saveFile ($fileName, $dataArray) {
		// Get output stream
		$outInstance = $this->getOutputStream();

		// Is it a valid stream?
		if (is_null($outInstance)) {
			// No class returned
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} elseif (!is_object($outInstance)) {
			// Not an object! ;-(
			throw new NoObjectException($outInstance, self::EXCEPTION_IS_NO_OBJECT);
		} elseif (!method_exists($outInstance, 'saveFile')) {
			// Nope, so throw exception
			throw new MissingMethodException(array($outInstance, 'saveFile'), self::EXCEPTION_MISSING_METHOD);
		}

		// Send the fileName and dataArray to the output handler
		$outInstance->saveFile($fileName, $dataArray);
	}

	/** Loads data from a file over the input handler
	 *
	 * @see	FileInputStreamer
	 */
	public function loadFileContents ($fqfn) {
		// Initialize the array
		$array = array();

		// Get output stream
		$inInstance = $this->getInputStream();

		// Is it a valid stream?
		if (is_null($inInstance)) {
			// No class returned
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} elseif (!is_object($inInstance)) {
			// Not an object! ;-(
			throw new NoObjectException($inInstance, self::EXCEPTION_IS_NO_OBJECT);
		} elseif (!method_exists($inInstance, 'loadFileContents')) {
			// Nope, so throw exception
			throw new MissingMethodException(array($inInstance, 'loadFileContents'), self::EXCEPTION_MISSING_METHOD);
		}

		// Read from the input handler
		return $inInstance->loadFileContents($fqfn);
	}
}

// [EOF]
?>
